@@ -58,14 +58,19 @@ class DefaultController(Controller):
             self.log().deactivate()
             self.log().activate()
             self.config().save()
-            self.review_controller().reset_but_try_to_keep_current_card()
+            self.review_controller().reset()
             self.next_rollover = self.database().start_of_day_n_days_ago(n=-1)
 
     def update_title(self):
         title = _("Mnemosyne")
-        database_name = self.database().display_name()
-        if database_name != self.database().default_name:
+        db = self.database()
+        database_name = db.display_name()
+        if database_name and database_name != db.default_name:
             title += " - " + database_name
+        if db.current_criterion() and \
+            db.current_criterion().name and \
+            db.current_criterion().name != db.default_criterion_name:
+            title += " - " + db.current_criterion().name
         self.main_widget().set_window_title(title)
 
     def show_add_cards_dialog(self):
@@ -138,8 +143,11 @@ class DefaultController(Controller):
                 if answer == 0:  # Do not add.
                     return
                 if answer == 2:  # Merge and edit.
-                    cards = card_type.create_sister_cards(fact)
                     db.add_fact(fact)
+                    for duplicate in duplicates:
+                        for card in db.cards_from_fact(duplicate):
+                            tags.update(card.tags)
+                    cards = card_type.create_sister_cards(fact)
                     for card in cards:
                         card.tags = tags
                         db.add_card(card)
@@ -158,10 +166,11 @@ class DefaultController(Controller):
                     self.delete_facts_and_their_cards(duplicates)
                     card = db.cards_from_fact(fact)[0]
                     card.fact.data = merged_fact_data
+                    card.tags = tags
                     self.component_manager.current("edit_card_dialog")\
                         (card, self.component_manager, allow_cancel=False).\
                         activate()
-                    return
+                    return db.cards_from_fact(fact)
         # Create cards.
         cards = card_type.create_sister_cards(fact)
         db.add_fact(fact)
@@ -182,9 +191,13 @@ class DefaultController(Controller):
         self.stopwatch().pause()
         self.flush_sync_server()
         review_controller = self.review_controller()
-        # This dialog calls 'edit_sister_cards' at some point.
-        self.component_manager.current("edit_card_dialog")\
+        # This dialog calls 'edit_card_and_sisters' at some point.
+        state = review_controller.state()
+        accepted = self.component_manager.current("edit_card_dialog")\
             (review_controller.card, self.component_manager).activate()
+        if not accepted:
+            self.stopwatch().unpause()
+            return
         review_controller.reload_counters()
         # Our current card could have disappeared from the database here,
         # e.g. when converting a front-to-back card to a cloze card, which
@@ -192,6 +205,7 @@ class DefaultController(Controller):
         if review_controller.card is None:
             review_controller.show_new_question()
         else:
+            review_controller.set_state(state)
             review_controller.card = self.database().card(\
                 review_controller.card._id, is_id_internal=True)
             # Our current card could have picked up a forbidden tag.
@@ -203,9 +217,11 @@ class DefaultController(Controller):
     def _change_card_type(self, fact, old_card_type, new_card_type,
                           correspondence, new_fact_data, warn=True):
 
-        """This is an internal function, used by 'edit_sister_cards' and
+        """This is an internal function, used by 'edit_card_and_sisters' and
         'change_card_type'. It should not be called from the outside by
         itself, otherwise the database will not be saved.
+        
+        Returns -2 for error, -1 for cancel and 0 for success.
 
         """
 
@@ -214,7 +230,11 @@ class DefaultController(Controller):
         db = self.database()
         cards_from_fact = db.cards_from_fact(fact)
         assert cards_from_fact[0].card_type == old_card_type
-        assert new_card_type.is_fact_data_valid(new_fact_data)
+        if not new_card_type.is_fact_data_valid(new_fact_data):
+            self.main_widget().show_error(\
+                _("Card data not correctly formatted for conversion.\n\nSkipping ") +\
+                "|".join(fact.data.values()) + ".\n")
+            return -2
         converter = self.component_manager.current\
               ("card_type_converter", used_for=(old_card_type.__class__,
                                                 new_card_type.__class__))
@@ -235,17 +255,26 @@ class DefaultController(Controller):
                 return 0
             else:
                 if warn:
-                    answer = self.main_widget().show_question(\
+                    has_history = False
+                    for card in cards_from_fact:
+                        if card.acq_reps > 0:
+                            has_history = True
+                            break
+                    if has_history:
+                        answer = self.main_widget().show_question(\
          _("Can't preserve history when converting between these card types.")\
                  + " " + _("The learning history of the cards will be reset."),
                  _("&OK"), _("&Cancel"), "")
-                    if answer == 1:  # Cancel.
-                        return -1
+                        if answer == 1:  # Cancel.
+                            return -1
                 # Go ahead with conversion
                 is_currently_asked = \
                    self.review_controller().card in cards_from_fact
                 tag_names = cards_from_fact[0].tag_string().split(", ")
-                self.delete_facts_and_their_cards([fact])
+                # Don't use progress bars in the next call, as nested
+                # progress bars (e.g. from change_card_type) are not
+                # supported.
+                self.delete_facts_and_their_cards([fact], progress_bar=False)
                 new_cards = self.create_new_cards(new_fact_data,
                     new_card_type, grade=-1, tag_names=tag_names)
                 # User cancelled in create_new_cards.
@@ -285,49 +314,62 @@ class DefaultController(Controller):
                 self.review_controller().reset()
             return 0
 
-    def edit_sister_cards(self, fact, new_fact_data, old_card_type,
-            new_card_type, new_tag_names, correspondence):
+    def edit_card_and_sisters(self, card, new_fact_data, new_card_type,
+            new_tag_names, correspondence):
         db = self.database()
         sch = self.scheduler()
         assert new_card_type.is_fact_data_valid(new_fact_data)
+        # Determine the current tags in use for the sister cards. This
+        # needs to be done before e.g. editing a cloze card creates new
+        # cards which are as yet untagged.
+        fact = db.fact(card.fact._id, is_id_internal=True)
+        current_sister_cards = self.database().cards_from_fact(fact)
+        current_tag_strings = set([sister_card.tag_string() \
+            for sister_card in current_sister_cards])        
         # Change the card type if needed. This does not take into account
         # changes to fact yet, which will come just afterwards.
-        result = self._change_card_type(fact, old_card_type, new_card_type,
-            correspondence, new_fact_data)
-        if result == -1:  # Aborted.
-            return -1
+        result = self._change_card_type(card.fact, card.card_type,
+            new_card_type, correspondence, new_fact_data)
+        if result in [-2, -1]:  # Error, aborted.
+            return result
         # When there was no card type conversion possible, the cards had to
         # be recreated from the new fact data. In that case, it is needed to
         # reload the fact from the database.
-        fact = db.fact(fact._id, is_id_internal=True)
+        fact = db.fact(card.fact._id, is_id_internal=True)
         # Update fact and create, delete and update cards.
         new_cards, edited_cards, deleted_cards = \
-            new_card_type.edit_sister_cards(fact, new_fact_data)
+            new_card_type.edit_fact(fact, new_fact_data)
         fact.data = new_fact_data
         db.update_fact(fact)
-        for card in deleted_cards:
-            if self.review_controller().card == card:
+        for deleted_card in deleted_cards:
+            if self.review_controller().card == deleted_card:
                 self.review_controller().card = None
-            sch.remove_from_queue_if_present(card)
-            db.delete_card(card)
-        for card in new_cards:
-            db.add_card(card)
-        for card in edited_cards:
-            db.update_card(card)
+            sch.remove_from_queue_if_present(deleted_card)
+            db.delete_card(deleted_card)
+        for new_card in new_cards:
+            db.add_card(new_card)
+        for edited_card in edited_cards:
+            db.update_card(edited_card)
         if new_cards and self.review_controller().learning_ahead == True:
             self.review_controller().reset()
         # Apply new tags and modification time to cards and save them back to
         # the database. Note that this makes sure there is an EDITED_CARD log
         # entry for each sister card, which is needed when syncing with a
         # partner that does not have the concept of facts.
+        tag_for_current_card_only = False
+        if len(current_tag_strings) > 1:
+            tag_for_current_card_only = bool(self.main_widget().show_question(
+            _("This card has different tags than its sister cards. Update tags for current card only or for all sister cards?"),
+            _("Current card only"), _("All sister cards"), "") == 0)
         old_tags = set()
         tags = db.get_or_create_tags_with_names(new_tag_names)
         modification_time = int(time.time())
-        for card in self.database().cards_from_fact(fact):
-            card.modification_time = modification_time
-            old_tags = old_tags.union(card.tags)
-            card.tags = tags
-            db.update_card(card)
+        for sister_card in self.database().cards_from_fact(fact):
+            sister_card.modification_time = modification_time
+            if sister_card == card or not tag_for_current_card_only:
+                old_tags = old_tags.union(sister_card.tags)
+                sister_card.tags = tags
+            db.update_card(sister_card)
         for tag in old_tags:
             db.delete_tag_if_unused(tag)
         db.save()
@@ -348,23 +390,37 @@ class DefaultController(Controller):
             if correspondence:
                 new_fact_data = {}
                 for old_fact_key, new_fact_key in correspondence.iteritems():
-                    new_fact_data[new_fact_key] = fact[old_fact_key]
-                assert new_card_type.is_fact_data_valid(new_fact_data)
-                fact.data = new_fact_data
+                    if old_fact_key in fact:
+                        new_fact_data[new_fact_key] = fact[old_fact_key]
+                if new_card_type.is_fact_data_valid(new_fact_data):
+                    fact.data = new_fact_data
             else:
-                new_fact_data = copy.copy(fact.data)
+                new_fact_data = copy.copy(fact.data)  
             result = self._change_card_type(fact, old_card_type,
-                new_card_type, correspondence, new_fact_data, warn)
-            if result == -1:
+                    new_card_type, correspondence, new_fact_data, warn)
+            if result == -1:  # Cancel.
                 w.close_progress()
-                return
-            if correspondence:
-                db.update_fact(fact)
+                return                    
+            if correspondence and result != -2:  # Error. 
+                db.update_fact(fact)              
             warn = False
             w.increase_progress(1)
         db.save()
         w.close_progress()
 
+    def star_current_card(self):
+        self.stopwatch().pause()
+        self.flush_sync_server()
+        db = self.database()
+        review_controller = self.review_controller()
+        tag = db.get_or_create_tag_with_name(_("Starred"))
+        db.add_tag_to_cards_with_internal_ids\
+            (tag, [review_controller.card._id])
+        review_controller.card = \
+            db.card(review_controller.card._id, is_id_internal=True)
+        review_controller.update_dialog(redraw_all=True)
+        self.stopwatch().unpause()
+
     def delete_current_card(self):
         self.stopwatch().pause()
         self.flush_sync_server()
@@ -383,9 +439,9 @@ class DefaultController(Controller):
                        + " " + _("sister cards?") + " " +\
                        _("Are you sure you want to do this,") + " " +\
           _("and not just deactivate cards in the 'Activate cards' dialog?")
-        answer = self.main_widget().show_question(question, _("&Delete"),
-                                          _("&Cancel"), "")
-        if answer == 1:  # Cancel.
+        answer = self.main_widget().show_question(question, _("&Cancel"),
+                                          _("&Delete"), "")
+        if answer == 0:  # Cancel.
             self.stopwatch().unpause()
             return
         self.delete_facts_and_their_cards([fact])
@@ -393,14 +449,33 @@ class DefaultController(Controller):
         review_controller.show_new_question()
         self.stopwatch().unpause()
 
-    def delete_facts_and_their_cards(self, facts):
+    def delete_facts_and_their_cards(self, facts, progress_bar=True):
+        assert len(facts) == len([fact.id for fact in facts])
         db = self.database()
+        w = self.main_widget()
+        if progress_bar:
+            w.set_progress_text(_("Deleting cards..."))
+            w.set_progress_range(len(facts))
+            w.set_progress_update_interval(50)
         for fact in facts:
             for card in db.cards_from_fact(fact):
                 self.scheduler().remove_from_queue_if_present(card)
-                db.delete_card(card)
+                db.delete_card(card, check_for_unused_tags=False)
             db.delete_fact(fact)
+            if progress_bar:
+                w.increase_progress(1)
+        tags = db.tags()
+        if progress_bar:
+            w.set_progress_text(_("Checking for unused tags..."))
+            w.set_progress_range(len(tags))
+        tags = db.tags()
+        for tag in tags:
+            db.delete_tag_if_unused(tag)
+            if progress_bar:
+                w.increase_progress(1)
         db.save()
+        if progress_bar:
+            w.close_progress()
 
     def clone_card_type(self, card_type, clone_name):
         from mnemosyne.libmnemosyne.utils import mangle
@@ -457,13 +532,14 @@ class DefaultController(Controller):
         self.database().update_card_type(card_type)
         self.database().save()
 
-    single_database_help = _("It is recommended to put all your cards in a single database. Using tags to determine which cards to study is much more convenient than having to load and unload several databases.")
+    single_database_help = \
+_("It is recommended to put all your cards in a single database. Using tags to determine which cards to study is much more convenient than having to load and unload several databases.")
 
     def show_new_file_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
         if self.config()["single_database_help_shown"] == False:
-            self.main_widget().show_information(_(self.single_database_help))
+            self.main_widget().show_information(self.single_database_help)
             self.config()["single_database_help_shown"] = True
         db = self.database()
         suffix = db.suffix
@@ -502,9 +578,13 @@ class DefaultController(Controller):
         if not filename:
             self.stopwatch().unpause()
             return
+        if filename.endswith(".cards"):
+            self.main_widget().show_information(\
+_("'*.cards' files are not separate databases, but need to be imported in your current database through 'File - Import'."))
+            return
         if filename.startswith(os.path.join(data_dir, "backups")):
             result = self.main_widget().show_question(\
-                _("Do you want to restore from this backup?\nNote that this will result in conflicts during the next sync, which need to be resolved by a full sync."),
+                _("Do you want to replace your current database with one restored from this backup?\nNote that this will result in conflicts during the next sync, which need to be resolved by a full sync."),
                 _("Yes"), _("No"), "")
             if result == 0:  # Yes.
                 # Note that we don't save the current database first in this
@@ -515,20 +595,23 @@ class DefaultController(Controller):
                 self.update_title()
             self.stopwatch().unpause()
             return
-        try:
-            self.log().saved_database()
-            db.backup()
-            db.unload()
-        except RuntimeError, error:
-            self.main_widget().show_error(unicode(error))
-            self.stopwatch().unpause()
-            return
+        if self.database().is_loaded():
+            try:
+                self.log().saved_database()
+                db.backup()
+                db.unload()
+            except RuntimeError, error:
+                self.main_widget().show_error(unicode(error.message))
+                self.stopwatch().unpause()
+                return
         try:
             db.load(filename)
             self.log().loaded_database()
             self.log().future_schedule()
-        except Exception, e:
-            self.main_widget().show_exception(e)
+        except Exception, error:
+            self.main_widget().show_error(unicode(error.message))
+            db.abandon()
+            db.load(old_path)
             self.stopwatch().unpause()
             return
         self.review_controller().reset()
@@ -538,11 +621,16 @@ class DefaultController(Controller):
     def save_file(self):
         self.stopwatch().pause()
         self.flush_sync_server()
+        if self.config()["save_database_help_shown"] == False:
+            self.main_widget().show_information(\
+_("Your database will be autosaved before exiting. Also, it is saved every couple of repetitions, as set in the configuration options."))
+            self.config()["save_database_help_shown"] = True
         try:
             self.database().save()
             self.log().saved_database()
+            self.main_widget().show_information(_("Database saved."))
         except RuntimeError, error:
-            self.main_widget().show_error(unicode(error))
+            self.main_widget().show_error(unicode(error.message))
         self.stopwatch().unpause()
 
     def show_save_file_as_dialog(self):
@@ -571,7 +659,7 @@ class DefaultController(Controller):
             shutil.copytree(old_media_dir, new_media_dir)
             self.log().saved_database()
         except RuntimeError, error:
-            self.main_widget().show_error(unicode(error))
+            self.main_widget().show_error(unicode(error.message))
             self.stopwatch().unpause()
             return
         self.review_controller().update_dialog()
@@ -599,7 +687,8 @@ class DefaultController(Controller):
         """
 
         from mnemosyne.libmnemosyne.utils import copy_file_to_dir
-        data_dir, media_dir = self.config().data_dir, self.database().media_dir()
+        data_dir, media_dir = \
+            self.config().data_dir, self.database().media_dir()
         path = expand_path(self.config()["import_img_dir"], data_dir)
         filter = _("Image files") + " " + filter
         filename = self.main_widget().get_filename_to_open(\
@@ -642,6 +731,31 @@ class DefaultController(Controller):
             filename = copy_file_to_dir(filename, media_dir)
             return filename
 
+    def show_insert_flash_dialog(self, filter):
+        from mnemosyne.libmnemosyne.utils import copy_file_to_dir
+        data_dir, media_dir = self.config().data_dir, self.database().media_dir()
+        path = expand_path(self.config()["import_flash_dir"], data_dir)
+        filter = _("Flash files") + " " + filter
+        filename = self.main_widget().get_filename_to_open(\
+            path, filter, _("Insert Flash"))
+        if not filename:
+            return ""
+        else:
+            self.config()["import_flash_dir"] = contract_path(\
+                os.path.dirname(filename), data_dir)
+            filename = copy_file_to_dir(filename, media_dir)
+            return filename
+
+    def show_browse_cards_dialog(self):
+        self.stopwatch().pause()
+        self.flush_sync_server()
+        review_controller = self.review_controller()
+        self.component_manager.current("browse_cards_dialog")\
+            (self.component_manager).activate()
+        review_controller.reset_but_try_to_keep_current_card()
+        review_controller.update_dialog(redraw_all=True)
+        self.stopwatch().unpause()
+
     def show_activate_cards_dialog(self):
         self.stopwatch().pause()
         self.flush_sync_server()
@@ -650,16 +764,20 @@ class DefaultController(Controller):
         review_controller = self.review_controller()
         review_controller.reset_but_try_to_keep_current_card()
         review_controller.update_status_bar_counters()
+        self.update_title()
         self.stopwatch().unpause()
 
-    def show_browse_cards_dialog(self):
+    def find_duplicates(self):
         self.stopwatch().pause()
         self.flush_sync_server()
-        self.component_manager.current("browse_cards_dialog")\
-            (self.component_manager).activate()
+        if self.config()["find_duplicates_help_shown"] == False:
+            self.main_widget().show_information(\
+_("This will tag all the cards in a given card type which have the same question. That way you can reformulate them to make the answer unambiguous. Note that this will not tag duplicates in different card types, e.g. card types for 'French' and 'Spanish'."))
+            self.config()["find_duplicates_help_shown"] = True
+        self.database().tag_all_duplicates()
         review_controller = self.review_controller()
         review_controller.reset_but_try_to_keep_current_card()
-        review_controller.update_dialog(redraw_all=True)
+        review_controller.update_status_bar_counters()
         self.stopwatch().unpause()
 
     def show_manage_plugins_dialog(self):
@@ -740,6 +858,7 @@ class DefaultController(Controller):
             os.remove(filename)
             if filename.endswith(".py") and os.path.exists(filename + "c"):
                 os.remove(filename + "c")
+        manifest.close()
         del plugin
         os.remove(manifest_filename)
         remove_empty_dirs_in(plugin_dir)
