@@ -5,10 +5,10 @@
 import os
 import sys
 import time
-import copy
 import string
 import shutil
 import datetime
+import copy as objcopy
 
 from openSM2sync.log_entry import EventTypes
 
@@ -19,7 +19,7 @@ from mnemosyne.libmnemosyne.translator import _
 from mnemosyne.libmnemosyne.database import Database
 from mnemosyne.libmnemosyne.card_type import CardType
 from mnemosyne.libmnemosyne.fact_view import FactView
-from mnemosyne.libmnemosyne.utils import traceback_string
+from mnemosyne.libmnemosyne.utils import traceback_string, copy
 from mnemosyne.libmnemosyne.utils import numeric_string_cmp, mangle
 from mnemosyne.libmnemosyne.utils import expand_path, contract_path
 
@@ -75,6 +75,7 @@ $pregenerated_data
     );
     create index i_cards on cards (id);
     create index i_cards_2 on cards (fact_view_id); /* for card type tree */
+    create index i_cards_3 on cards (_fact_id); /* for cards_from_fact */
 
     create table tags(
         _id integer primary key,
@@ -219,7 +220,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
 
     """
 
-    version = "Mnemosyne SQL 1.0"
+    version = "2"
     suffix = ".db"
     store_pregenerated_data = True
 
@@ -271,17 +272,29 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         return self._path
 
     def name(self):
-        return os.path.basename(self.config()["last_database"])
+        return os.path.basename(self._path)
 
     def display_name(self):
         if not self.is_loaded():
             return None
         else:
-            return os.path.basename(self.config()["last_database"]).\
-                   split(self.database().suffix)[0]
+            return os.path.basename(self._path).\
+                split(self.database().suffix)[0]
 
     def compact(self):
         self.con.execute("vacuum")
+        # Make sure the "Untagged" tag does not show up together with
+        # different tags.
+        untagged = self.tag("__UNTAGGED__", is_id_internal=False)
+        for cursor in self.con.execute("select _id from cards"):
+            _card_id = cursor[0]
+            _tag_ids = [cursor2[0] for cursor2 in self.con.execute(\
+                "select _tag_id from tags_for_card where _card_id=?",
+                (_card_id, ))]
+            if len(_tag_ids) > 1 and untagged._id in _tag_ids:
+                self.con.execute(\
+                    "delete from tags_for_card where _card_id=? and _tag_id=?",
+                    (_card_id, untagged._id))
 
     def new(self, path):
         self.unload()
@@ -303,7 +316,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
             values(?,?)""", ("log.txt", 0))
         self.config()["last_database"] = \
             contract_path(self._path, self.config().data_dir)
-        # Create __UNTAGGED__ tag
+        # Create __UNTAGGED__ tag.
         tag = Tag("__UNTAGGED__", "__UNTAGGED__")
         self.add_tag(tag)
         # Create default criterion.
@@ -312,7 +325,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         self._current_criterion = DefaultCriterion(self.component_manager)
         self._current_criterion._id = 1
         self._current_criterion.id = "__DEFAULT__"
-        self._current_criterion.name = "__DEFAULT__"
+        self._current_criterion.name = self.default_criterion_name
         self._current_criterion._tag_ids_active.add(tag._id)
         self.add_criterion(self._current_criterion)
 
@@ -336,18 +349,30 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         if self.is_loaded():
             self.unload()
         self._path = expand_path(path, self.config().data_dir)
-        self.create_media_dir_if_needed()
         # Check database version.
         try:
             sql_res = self.con.execute("""select value from global_variables
                 where key=?""", ("version", )).fetchone()
         except:
-            raise RuntimeError, _("Unable to load file.") + traceback_string()
+            raise RuntimeError, _("Unable to load file at") + " " + self._path
         if sql_res is None:
-            raise RuntimeError, _("Unable to load file, query failed")
+            raise RuntimeError, _("Unable to load file, query failed.")
         if sql_res[0] != self.version:
-            raise RuntimeError, \
-                _("Unable to load file: database version mismatch.")
+            if sql_res[0] == "Mnemosyne SQL 1.0":
+                previous_version = 1
+            else:
+                previous_version = int(sql_res[0])
+            try:
+                if previous_version <= 2:
+                    from mnemosyne.libmnemosyne.upgrades.upgrade2 \
+                        import Upgrade2
+                    Upgrade2(self.component_manager).run()
+            except:
+                raise RuntimeError, _("Database upgrade failed.")
+        self.create_media_dir_if_needed()
+        # Upgrade.
+        self.con.execute("""create index if not exists
+            i_cards_3 on cards (_fact_id);""")
         # Identify missing plugins for card types and their parents.
         plugin_needed_ids = set()
         builtin_ids = set(card_type.id for card_type in self.card_types())
@@ -400,7 +425,13 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
             return
         dest_path = expand_path(path, self.config().data_dir)
         if dest_path != self._path:
-            shutil.copy(self._path, dest_path)
+            if sys.platform == "win32":  # pragma: no cover
+                drive = os.path.splitdrive(path)[0]
+                import ctypes
+                if ctypes.windll.kernel32.GetDriveTypeW(u"%s\\" % drive) == 4:
+                    raise RuntimeError, \
+_("Putting a database on a network drive is forbidden under Windows to avoid data corruption.")
+            copy(self._path, dest_path)
             self._path = dest_path
         self.config()["last_database"] \
             = contract_path(path, self.config().data_dir)
@@ -418,13 +449,14 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         backupfile = os.path.join(backupdir, backupfile)
         failed = False
         try:
-            shutil.copy(self._path, backupfile)
+            copy(self._path, backupfile)
         except:
             failed = True
         if failed or not os.path.exists(backupfile) or \
           not os.stat(backupfile).st_size:
             self.main_widget().show_information(\
                 _("Warning: backup creation failed for") + " " +  backupfile)
+            return None
         for f in self.component_manager.all("hook", "after_backup"):
             f.run(backupfile)
         # Only keep the last logs.
@@ -443,7 +475,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         self.abandon()
         db_path = expand_path(\
             self.config()["last_database"], self.config().data_dir)
-        shutil.copy(path, db_path)
+        copy(path, db_path)
         self.load(db_path)
         # We need to indicate that a full sync needs to happen on the next
         # sync. Unfortunately, we can't do anything about the logs that have
@@ -522,6 +554,8 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
 
     def get_or_create_tag_with_name(self, name):
         name = name.strip()
+        if name.startswith("::"):
+            name = name[2:]
         sql_res = self.con.execute("""select _id, id, name, extra_data from
             tags where name=?""", (name, )).fetchone()
         if sql_res:
@@ -565,12 +599,32 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
                 criterion.id != "__DEFAULT__":
                 saved_criterion = criterion
                 break
+        # If there is no explictly named criterion, we always activate the
+        # tag except when a parent is inactive.
         if not saved_criterion:
             criteria_to_activate_tag_in = [current_criterion]
+            existing_tag_for_name = {}
+            for _tag in self.tags():
+                existing_tag_for_name[_tag.name] = _tag
+            partial_tag_name = ""
+            for node in tag.name.split("::"):
+                partial_tag_name += node
+                if partial_tag_name != tag.name and \
+                    partial_tag_name in existing_tag_for_name:
+                    parent = existing_tag_for_name[partial_tag_name]
+                    if not current_criterion.is_tag_active(parent):
+                        criteria_to_activate_tag_in = []
+                        break
+                partial_tag_name += "::"
+        # If there is a saved criterion active, we ask the user what to do.
         else:
-            answer = self.main_widget().show_question(\
-                _("Make tag '%s' active in saved set '%s'?") % \
-                (tag.name, saved_criterion.name), _("Yes"), _("No"), "")
+            try:
+                answer = self.main_widget().show_question(\
+                    _("Make tag '%s' active in saved set '%s'?") % \
+                    (tag.name, saved_criterion.name), _("Yes"), _("No"), "")
+            except NotImplementedError: 
+                # We are running in a non interactive mode.
+                answer = 0  # Yes
             if answer == 1:  # No.
                 criteria_to_activate_tag_in = []
             else:
@@ -896,12 +950,13 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
             self.con.execute("""insert into tags_for_card(_tag_id,
                 _card_id) values(?,?)""", (tag._id, card._id))
 
-    def delete_card(self, card):
+    def delete_card(self, card, check_for_unused_tags=True):
         self.con.execute("delete from cards where _id=?", (card._id, ))
         self.con.execute("delete from tags_for_card where _card_id=?",
             (card._id, ))
-        for tag in card.tags:
-            self.delete_tag_if_unused(tag)
+        if not self.syncing and check_for_unused_tags:
+            for tag in card.tags:
+                self.delete_tag_if_unused(tag)
         self.log().deleted_card(card)
         del card
 
@@ -947,6 +1002,7 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
                 (EventTypes.EDITED_CARD, int(time.time()), card_id))
 
     def remove_tag_from_cards_with_internal_ids(self, tag, _card_ids):
+        # Delete tags.
         arguments = ((tag._id, _card_id) for _card_id in _card_ids)
         self.con.executemany("""delete from tags_for_card where _tag_id=?
             and _card_id=?""", arguments)
@@ -1189,10 +1245,9 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
         del criterion
 
     def set_current_criterion(self, criterion):
-        criterion = copy.copy(criterion)
+        criterion = objcopy.copy(criterion)
         criterion._id = 1
         criterion.id = "__DEFAULT__"
-        criterion.name = "__DEFAULT__"
         self.update_criterion(criterion)
         applier = self.component_manager.current("criterion_applier",
             used_for=criterion.__class__)
@@ -1244,6 +1299,60 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
                 facts.append(self.fact(_fact_id, is_id_internal=True))
         return facts
 
+    def tag_all_duplicates(self):
+        # Find the _fact_ids of the candidate duplicates, i.e. not yet taking
+        # into account that duplicates in different card types are allowed and
+        # that we only need to check the unique fact keys. This preprocessing
+        # step speeds up the rest of the calculations.
+        _fact_ids = set()
+        _fact_id_for_data = {}
+        for key, value, _fact_id in self.con.execute(\
+            """select key, value, _fact_id from data_for_fact"""):
+            if key not in _fact_id_for_data:
+                _fact_id_for_data[key] = {}
+            if value in _fact_id_for_data[key]:
+                _fact_ids.add(_fact_id_for_data[key][value])
+                _fact_ids.add(_fact_id)
+            else:
+                _fact_id_for_data[key][value] = _fact_id
+        # Sort the candidates per card type.
+        _fact_ids_in_card_type = {}
+        for _fact_id in _fact_ids:
+            card_type_id = self.con.execute("""select card_type_id from cards
+                where _fact_id=?""", (_fact_id, )).fetchone()[0]
+            if card_type_id not in _fact_ids_in_card_type:
+                _fact_ids_in_card_type[card_type_id] = []
+            _fact_ids_in_card_type[card_type_id].append(_fact_id)
+        # Check if the duplicates are really in the fields that should be
+        # unique.
+        duplicate_facts = set([])
+        for card_type_id in _fact_ids_in_card_type:
+            facts = [self.fact(_fact_id, is_id_internal=True) for _fact_id in \
+                _fact_ids_in_card_type[card_type_id]]
+            for fact_key in self.card_type_with_id(card_type_id).\
+                unique_fact_keys:
+                for i in range(len(facts)):
+                    for j in range(i + 1, len(facts)):
+                        if facts[i][fact_key] == facts[j][fact_key]:
+                            duplicate_facts.add(facts[i])
+                            duplicate_facts.add(facts[j])
+        # Tag the duplicate cards.
+        _card_ids = []
+        for fact in duplicate_facts:
+            _card_ids += [cursor[0] for cursor in self.con.execute(\
+                "select _id from cards where _fact_id=?", (fact._id, ))]
+        if len(_card_ids) == 0:
+            self.main_widget().show_information(_("No duplicates found."))
+        else:
+            self.main_widget().show_information(\
+ _("Found %d duplicate cards. They have been given the tag 'DUPLICATE'.") \
+            % (len(_card_ids), ))
+        # We add the tags after we showed the dialog box, as adding tags can
+        # trigger a dialog asking if DUPLICATE should be made active in the
+        # current set.
+        self.add_tag_to_cards_with_internal_ids(\
+            self.get_or_create_tag_with_name(_("DUPLICATE")), _card_ids)
+
     def card_types_in_use(self):
         return [self.card_type_with_id(cursor[0]) for cursor in \
             self.con.execute ("select distinct card_type_id from cards")]
@@ -1299,8 +1408,9 @@ class SQLite(Database, SQLiteSync, SQLiteMedia, SQLiteLogging,
             # Make sure they are truly duplicates, and not coming
             # from two values for the same key in 'fact_id_for_front' and
             # 'fact_id_for_back'.
-            if card_1.fact["f"] != card_2.fact["b"] or \
-                card_1.fact["b"] != card_2.fact["f"]:
+            if "b" in card_1.fact.data and "b" in card_2.fact.data and \
+                (card_1.fact["f"] != card_2.fact["b"] or \
+                card_1.fact["b"] != card_2.fact["f"]):
                 continue
             # Tags should be equal.
             if card_1.tag_string() != card_2.tag_string():
