@@ -19,7 +19,8 @@ from mnemosyne.libmnemosyne.utils import expand_path
 from mnemosyne.libmnemosyne.card_type import CardType
 from mnemosyne.libmnemosyne.fact_view import FactView
 
-re_src = re.compile(r"""src=['\"](.+?)['\"]""", re.DOTALL | re.IGNORECASE)
+re_src = re.compile(r"""(src|data)=['\"](.+?)['\"]""", 
+                    re.DOTALL | re.IGNORECASE)
 
 # Simple named-tuple like class, to avoid the expensive creation a full card
 # object (Python 2.5 does not yet have a named tuple).
@@ -201,6 +202,16 @@ class SQLiteSync(object):
                 filenames.add(filename)
         return filenames
 
+    def generate_log_entries_for_settings(self):
+
+        """Needed after binary initial upload/download of the database, to
+        ensure that the side effects to config get applied.
+
+        """
+
+        for key in self.config().keys_to_sync:
+            self.log().edited_setting(key)
+
     def active_objects_to_export(self):
         active_objects = {}
         # Active facts (working with python sets turns out to be more
@@ -244,13 +255,12 @@ class SQLiteSync(object):
                 "select fact_view_ids from card_types where id=?",
                 (card_type_id, )).fetchone()[0])
         # Media files for active cards.
-        self.dynamically_create_media_files()
-        active_objects["media_filenames"] = set()
+        active_objects["media_filenames"] = self.active_dynamic_media_files()
         for result in self.con.execute(\
             """select value from data_for_fact where _fact_id in (select
             _fact_id from cards where active=1) and value like '%src=%'"""):
             for match in re_src.finditer(result[0]):
-                active_objects["media_filenames"].add(match.group(1))
+                active_objects["media_filenames"].add(match.group(2))
         return active_objects
 
     def set_extra_tags_on_import(self, tags):
@@ -322,6 +332,8 @@ class SQLiteSync(object):
         elif event_type in (EventTypes.ADDED_TAG, EventTypes.EDITED_TAG):
             try:
                 tag = self.tag(log_entry["o_id"], is_id_internal=False)
+                if tag is None:
+                    return None
                 log_entry["name"] = tag.name
                 if tag.extra_data:
                     log_entry["extra"] = repr(tag.extra_data)
@@ -390,6 +402,8 @@ class SQLiteSync(object):
             except TypeError: # The object has been deleted at a later stage.
                 pass
         elif event_type == EventTypes.EDITED_SETTING:
+            if log_entry["o_id"] not in self.config():  # Obsolete entry.
+                return None
             log_entry["value"] = repr(self.config()[log_entry["o_id"]])
         return log_entry
 
@@ -578,6 +592,9 @@ class SQLiteSync(object):
                 # EDITED_CARD event.
                 pass
         if self.importing:
+            if len(self.extra_tags_on_import) != 0:
+                card.tags.discard(\
+                    self.tag("__UNTAGGED__", is_id_internal=False))
             for tag in self.extra_tags_on_import:
                 card.tags.add(tag)
         # Construct rest of card. The 'active' property does not need to be
@@ -670,7 +687,11 @@ class SQLiteSync(object):
                 log_entry["type"] = EventTypes.EDITED_FACT_VIEW
                 return self.update_fact_view(\
                     self.fact_view_from_log_entry(log_entry))
-        self.add_fact_view(self.fact_view_from_log_entry(log_entry))
+        try:
+            self.add_fact_view(self.fact_view_from_log_entry(log_entry))
+        except:
+            # Leftover from old bug.
+            print "Creating same fact view twice during sync."
 
     def fact_view_from_log_entry(self, log_entry):
         # Get fact view object to be deleted now.
@@ -712,7 +733,11 @@ class SQLiteSync(object):
                 log_entry["type"] = EventTypes.EDITED_CARD_TYPE
                 return self.update_card_type(\
                     self.card_type_from_log_entry(log_entry))
-        self.add_card_type(self.card_type_from_log_entry(log_entry))
+        try:
+            self.add_card_type(self.card_type_from_log_entry(log_entry))
+        except:
+            # Leftover from old bug.
+            print "Creating same card type twice during sync."
 
     def card_type_from_log_entry(self, log_entry):
         # Get card type object to be deleted now.
@@ -847,11 +872,14 @@ class SQLiteSync(object):
                 key, value = log_entry["o_id"], eval(log_entry["value"])
                 if key in self.config().keys_to_sync:
                     self.config()[key] = value
+                    for card_type in self.card_types():
+                        for render_chain in self.component_manager.\
+                            all("render_chain"):
+                            render_chain.renderer_for_card_type(card_type).\
+                                update(card_type)                    
                 else:
                     self.log_edited_setting(log_entry["time"], key)
         finally:
             self.log().timestamp = None
             self.syncing = False
             self.importing = False
-
-
