@@ -29,11 +29,13 @@ class AddEditCards(TipAfterStartingNTimes):
          6: _("You can organise tags in a hierarchy by using :: as separator, e.g. My book::Lesson 1."),
          9: _("You can add images and sounds to your cards. Right-click on an input field when editing a card to bring up a pop-up menu to do so."),
          12: _("If for a certain card type cloned from Vocabulary you don't need a pronunciation field, you can hide it by right-clicking on it and using the pop-up menu."),
-         15: _("You can use Tab to move between the fields. For 'Add cards', use Ctrl+Enter for 'Yet to learn', and Ctrl+2, etc. for the grades. For 'Edit card', use Ctrl-Enter to close.")}
+         15: _("You can use Tab to move between the fields. For 'Add cards', use Ctrl+Enter for 'Yet to learn', and Ctrl+2, etc. for the grades. For 'Edit card', use Ctrl-Enter to close."),
+         18: _("If you use 'Edit cards', changes are made to all the sister cards simultaneously.")}
 
     def activate(self):
         self.show_tip_after_starting_n_times()
-        self.exec_()
+        status = self.exec_()
+        return (status == QtGui.QDialog.Accepted)
 
     def initialise_card_types_combobox(self, current_card_type_name):
         # We calculate card_type_by_name here because these names can change
@@ -131,10 +133,14 @@ class AddEditCards(TipAfterStartingNTimes):
             self.config()["last_used_tags_for_card_type_id"]:
             self.config()["last_used_tags_for_card_type_id"]\
                 [new_card_type.id] = ""
-        if not unicode(self.tags.currentText()) \
-            or self.card_type_widget.is_empty():
-            self.update_tags_combobox(self.config()\
-                ["last_used_tags_for_card_type_id"][new_card_type.id])
+        if not self.config()["is_last_used_tags_per_card_type"]:
+            if not unicode(self.tags.currentText()):
+                self.update_tags_combobox(self.config()["last_used_tags"])
+        else:
+            if not unicode(self.tags.currentText()) \
+                or self.card_type_widget.is_empty():
+                self.update_tags_combobox(self.config()\
+                    ["last_used_tags_for_card_type_id"][new_card_type.id])
         if self.card_type.fact_keys().issubset(new_card_type.fact_keys()) or \
             self.card_type_widget.is_empty():
             self.update_card_widget()
@@ -142,6 +148,10 @@ class AddEditCards(TipAfterStartingNTimes):
         dlg = ConvertCardTypeKeysDlg(self.card_type, new_card_type,
             self.correspondence, check_required_fact_keys=False, parent=self)
         if dlg.exec_() != QtGui.QDialog.Accepted:
+            # Set correspondence so as not to erase previous data.
+            self.correspondence = {}
+            for key in self.card_type.fact_keys():
+                self.correspondence[key] = key
             self.card_types_widget.setCurrentIndex(self.card_type_index)
             return
         else:
@@ -181,8 +191,11 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
             self.config()["last_used_tags_for_card_type_id"]:
             self.config()["last_used_tags_for_card_type_id"]\
                 [last_used_card_type.id] = ""
-        self.update_tags_combobox(self.config()\
-           ["last_used_tags_for_card_type_id"][last_used_card_type.id])
+        if not self.config()["is_last_used_tags_per_card_type"]:
+            self.update_tags_combobox(self.config()["last_used_tags"])
+        else:
+            self.update_tags_combobox(self.config()\
+                ["last_used_tags_for_card_type_id"][last_used_card_type.id])
         self.grades = QtGui.QButtonGroup()
         # Negative indexes have special meanings in Qt, so we can't use -1 for
         # 'yet to learn'.
@@ -198,12 +211,11 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
             self.restoreGeometry(state)
 
     def keyPressEvent(self, event):
-        # Note: for the following to work reliably, there should be no
-        # shortcuts defined in the ui file.
         if self.yet_to_learn_button.isEnabled() and event.modifiers() in \
             [QtCore.Qt.ControlModifier, QtCore.Qt.AltModifier]:
             if event.key() in [QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return,
                 QtCore.Qt.Key_Y, QtCore.Qt.Key_0, QtCore.Qt.Key_1]:
+                # Qt bug: works for Enter, Return, but not 0 or 1.
                 self.create_new_cards(-1)
             elif event.key() == QtCore.Qt.Key_2:
                 self.create_new_cards(2)
@@ -236,6 +248,7 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
         c.create_new_cards(fact_data, card_type, grade, tag_names, save=True)
         tag_text = ", ".join(tag_names)
         self.update_tags_combobox(tag_text)
+        self.config()["last_used_tags"] = tag_text
         self.config()["last_used_tags_for_card_type_id"][card_type.id] \
             = tag_text
         self.card_type_widget.clear()
@@ -243,8 +256,8 @@ class AddCardsDlg(QtGui.QDialog, Ui_AddCardsDlg, AddEditCards, AddCardsDialog):
     def reject(self):
         if not self.card_type_widget.is_empty() and \
             self.card_type_widget.is_changed():
-            status = QtGui.QMessageBox.warning(None, _("Mnemosyne"),
-                _("Abandon current card?"), _("&Yes"), _("&No"), "", 1, -1)
+            status = self.main_widget().show_question(\
+                _("Abandon current card?"), _("&Yes"), _("&No"), "")
             if status == 0:
                 QtGui.QDialog.reject(self)
                 return
