@@ -61,10 +61,15 @@ class CardModel(QtSql.QSqlTableModel, Component):
         self.background_colour_for_card_type_id = {}
         for card_type_id, rgb in \
             self.config()["background_colour"].iteritems():
+            # If the card type has been deleted since, don't bother.
+            if not card_type_id in self.component_manager.card_type_with_id:
+                continue
             self.background_colour_for_card_type_id[card_type_id] = \
                 QtGui.QColor(rgb)
         self.font_colour_for_card_type_id = {}
         for card_type_id in self.config()["font_colour"]:
+            if not card_type_id in self.component_manager.card_type_with_id:
+                continue
             first_key = \
                 self.card_type_with_id(card_type_id).fact_keys_and_names[0][0]
             self.font_colour_for_card_type_id[card_type_id] = QtGui.QColor(\
@@ -87,7 +92,8 @@ class CardModel(QtSql.QSqlTableModel, Component):
                 return QtCore.QVariant(\
                     self.background_colour_for_card_type_id[card_type_id])
             else:
-                return QtCore.QVariant(QtGui.QFont("white"))
+                return QtCore.QVariant(\
+                    QtGui.qApp.palette().color(QtGui.QPalette.Base))
         column = index.column()
         if role == QtCore.Qt.TextAlignmentRole and column not in \
             (QUESTION, ANSWER, TAGS):
@@ -113,13 +119,13 @@ class CardModel(QtSql.QSqlTableModel, Component):
                 return QtCore.QVariant(grade)
         if column == NEXT_REP:
             next_rep = QtSql.QSqlTableModel.data(self, index, role).toInt()[0]
-            if next_rep == -1:
+            if next_rep <= 0:
                 return QtCore.QVariant("")
             return QtCore.QVariant(\
                 self.scheduler().next_rep_to_interval_string(next_rep))
         if column == LAST_REP:
             last_rep = QtSql.QSqlTableModel.data(self, index, role).toInt()[0]
-            if last_rep == -1:
+            if last_rep <= 0:
                 return QtCore.QVariant("")
             return QtCore.QVariant(\
                 self.scheduler().last_rep_to_interval_string(last_rep))
@@ -129,7 +135,7 @@ class CardModel(QtSql.QSqlTableModel, Component):
         if column in (CREATION_TIME, MODIFICATION_TIME):
             old_data = QtSql.QSqlTableModel.data(self, index, role).toInt()[0]
             return QtCore.QVariant(time.strftime(self.date_format,
-                time.gmtime(old_data)))
+                time.localtime(old_data)))
         return QtSql.QSqlTableModel.data(self, index, role)
 
 
@@ -263,7 +269,8 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
          9 : _("You can reorder columns in the card browser by dragging the header label."),
         12 : _("You can resize columns in the card browser by dragging between the header labels."),
         15 : _("When editing or previewing cards from the card browser, PageUp/PageDown can be used to move to the previous/next card."),
-        18 : _("You change the relative size of the card list, card type tree and tag tree by dragging the dividers between them.")}
+        18 : _("You change the relative size of the card list, card type tree and tag tree by dragging the dividers between them."),
+        21 : _("In the search box, you can use SQL wildcards like _ (matching a single character) and % (matching one or more characters).")}
 
     def __init__(self, component_manager):
         BrowseCardsDialog.__init__(self, component_manager)
@@ -275,6 +282,7 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         self.setWindowFlags(self.windowFlags() \
             & ~ QtCore.Qt.WindowContextHelpButtonHint)
         self.saved_index = None
+        self.card_model = None
         # Set up card type tree.
         self.container_1 = QtGui.QWidget(self.splitter_1)
         self.layout_1 = QtGui.QVBoxLayout(self.container_1)
@@ -282,8 +290,10 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             self.container_1)
         self.layout_1.addWidget(self.label_1)
         self.card_type_tree_wdgt = \
-            CardTypesTreeWdgt(component_manager, self.container_1,
-                self.unload_qt_database, self.display_card_table)
+            CardTypesTreeWdgt(component_manager, self.container_1, 
+            self.unload_qt_database)
+        self.card_type_tree_wdgt.card_types_changed_signal.\
+            connect(self.reload_database_and_redraw)
         self.layout_1.addWidget(self.card_type_tree_wdgt)
         self.splitter_1.insertWidget(0, self.container_1)
         # Set up tag tree plus search box.
@@ -294,7 +304,9 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         self.layout_2.addWidget(self.label_2)
         self.tag_tree_wdgt = \
             TagsTreeWdgt(component_manager, self.container_2,
-                 self.unload_qt_database, self.display_card_table)
+            self.unload_qt_database)
+        self.tag_tree_wdgt.tags_changed_signal.\
+            connect(self.reload_database_and_redraw) 
         self.layout_2.addWidget(self.tag_tree_wdgt)
         self.label_3 = QtGui.QLabel(_("containing this text in the cards:"),
             self.container_2)
@@ -314,6 +326,7 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         # When starting the widget, we default with the current criterion
         # as filter. In this case, we can make a shortcut simply by selecting
         # on 'active=1'
+        self.load_qt_database()
         self.display_card_table(run_filter=False)
         self.card_model.setFilter("cards.active=1")
         self.card_model.select()
@@ -339,6 +352,10 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             self.splitter_2.setSizes([333, 630])
         else:
             self.splitter_2.restoreState(splitter_2_state)
+        for column in (_ID, ID, CARD_TYPE_ID, _FACT_ID, FACT_VIEW_ID,
+            ACQ_REPS_SINCE_LAPSE, RET_REPS_SINCE_LAPSE,
+            EXTRA_DATA, ACTIVE, SCHEDULER_DATA):
+            self.table.setColumnHidden(column, True)
 
     def context_menu(self, point):
         menu = QtGui.QMenu(self)
@@ -374,7 +391,7 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
 
     def keyPressEvent(self, event):
         if len(self.table.selectionModel().selectedRows()) == 0:
-            return QtGui.QDialog.keyPressEvent(self, event)
+            QtGui.QDialog.keyPressEvent(self, event)
         if event.key() in [QtCore.Qt.Key_Enter, QtCore.Qt.Key_Return]:
             self.menu_edit()
         elif event.key() == QtCore.Qt.Key_E and \
@@ -383,13 +400,19 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         elif event.key() == QtCore.Qt.Key_P and \
             event.modifiers() == QtCore.Qt.ControlModifier:
             self.menu_preview()
+        elif event.key() == QtCore.Qt.Key_F and \
+            event.modifiers() == QtCore.Qt.ControlModifier:
+            self.search_box.setFocus()
         elif event.key() in [QtCore.Qt.Key_Delete, QtCore.Qt.Key_Backspace]:
             self.menu_delete()
         else:
             QtGui.QDialog.keyPressEvent(self, event)
 
-    def cards_from_single_selection(self):
-        index = self.table.selectionModel().selectedRows()[0]
+    def sister_cards_from_single_selection(self):
+        selected_rows = self.table.selectionModel().selectedRows()
+        if len(selected_rows) == 0:
+            return []
+        index = selected_rows[0]
         _fact_id_index = index.model().index(\
             index.row(), _FACT_ID, index.parent())
         _fact_id = index.model().data(_fact_id_index).toInt()[0]
@@ -420,15 +443,22 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
     def menu_edit(self, index=None):
         # 'index' gets passed if this function gets called through the
         # table.doubleClicked event.
-        cards = self.cards_from_single_selection()
+        _card_ids = self._card_ids_from_selection()
+        if len(_card_ids) == 0:
+            return
+        card = self.database().card(_card_ids.pop(), is_id_internal=True)
         self.edit_dlg = self.component_manager.current("edit_card_dialog")\
-            (cards[0], self.component_manager, started_from_card_browser=True)
+            (card, self.component_manager, started_from_card_browser=True,
+            parent=self)
+        # Here, we don't unload the database already by ourselves, but leave
+        # it to the edit dialog to only do so if needed.
         self.edit_dlg.before_apply_hook = self.unload_qt_database
         self.edit_dlg.page_up_down_signal.connect(self.page_up_down_edit)
         if self.edit_dlg.exec_() == QtGui.QDialog.Accepted:
-            self.display_card_table()
             self.card_type_tree_wdgt.rebuild()
             self.tag_tree_wdgt.rebuild()
+            self.load_qt_database()
+            self.display_card_table()
         # Avoid multiple connections.
         self.edit_dlg.page_up_down_signal.disconnect(self.page_up_down_edit)
 
@@ -439,11 +469,13 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         elif up_down == self.edit_dlg.DOWN:
             shift = 1
         self.table.selectRow(current_row + shift)
-        self.edit_dlg.set_new_card(self.cards_from_single_selection()[0])
+        _card_ids = self._card_ids_from_selection()
+        card = self.database().card(_card_ids.pop(), is_id_internal=True)
+        self.edit_dlg.set_new_card(card)
 
     def menu_preview(self):
         from mnemosyne.pyqt_ui.preview_cards_dlg import PreviewCardsDlg
-        cards = self.cards_from_single_selection()
+        cards = self.sister_cards_from_single_selection()
         tag_text = cards[0].tag_string()
         self.preview_dlg = \
             PreviewCardsDlg(self.component_manager, cards, tag_text, self)
@@ -463,7 +495,7 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             shift = 1
         self.table.selectRow(current_row + shift)
         self.preview_dlg.index = 0
-        self.preview_dlg.cards = self.cards_from_single_selection()
+        self.preview_dlg.cards = self.sister_cards_from_single_selection()
         self.preview_dlg.tag_text = self.preview_dlg.cards[0].tag_string()
         self.preview_dlg.update_dialog()
 
@@ -473,18 +505,22 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             _("&OK"), _("&Cancel"), "")
         if answer == 1: # Cancel.
             return
-        facts = []
+        _fact_ids = set()
         for index in self.table.selectionModel().selectedRows():
             _fact_id_index = index.model().index(\
                 index.row(), _FACT_ID, index.parent())
             _fact_id = index.model().data(_fact_id_index).toInt()[0]
+            _fact_ids.add(_fact_id)
+        facts = []
+        for _fact_id in _fact_ids:
             facts.append(self.database().fact(_fact_id, is_id_internal=True))
         self.unload_qt_database()
         self.saved_selection = []
         self.controller().delete_facts_and_their_cards(facts)
-        self.display_card_table()
         self.card_type_tree_wdgt.rebuild()
         self.tag_tree_wdgt.rebuild()
+        self.load_qt_database()
+        self.display_card_table()
 
     def menu_change_card_type(self):
         # Test if all selected cards have the same card type.
@@ -521,16 +557,21 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         self.unload_qt_database()
         self.controller().change_card_type(facts, current_card_type,
             new_card_type, self.correspondence)
-        self.display_card_table()
         self.card_type_tree_wdgt.rebuild()
         self.tag_tree_wdgt.rebuild()
+        self.load_qt_database()
+        self.display_card_table()
 
     def menu_add_tags(self):
+        if not self.config()["showed_help_on_adding_tags"]:
+            self.main_widget().show_information(\
+"With this option, can you edit the tags of individual cards, without affecting sister cards.")
+            self.config()["showed_help_on_adding_tags"] = True
         # Get new tag names. Use a dict as backdoor to return values
         # from the dialog.
         return_values = {}
         from mnemosyne.pyqt_ui.add_tags_dlg import AddTagsDlg
-        dlg = AddTagsDlg(self.component_manager, return_values)
+        dlg = AddTagsDlg(self.component_manager, return_values, parent=self)
         if dlg.exec_() != QtGui.QDialog.Accepted:
             return
         # Add the tags.
@@ -541,10 +582,15 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
                 continue
             tag = self.database().get_or_create_tag_with_name(tag_name)
             self.database().add_tag_to_cards_with_internal_ids(tag, _card_ids)
-        self.display_card_table()
         self.tag_tree_wdgt.rebuild()
+        self.load_qt_database()
+        self.display_card_table()
 
     def menu_remove_tags(self):
+        if not self.config()["showed_help_on_adding_tags"]:
+            self.main_widget().show_information(\
+"With this option, can you edit the tags of individual cards, without affecting sister cards.")
+            self.config()["showed_help_on_adding_tags"] = True
         # Figure out the tags used by the selected cards.
         _card_ids = self._card_ids_from_selection()
         tags = self.database().tags_from_cards_with_internal_ids(_card_ids)
@@ -563,8 +609,9 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             tag = self.database().get_or_create_tag_with_name(tag_name)
             self.database().remove_tag_from_cards_with_internal_ids(\
                 tag, _card_ids)
-        self.display_card_table()
         self.tag_tree_wdgt.rebuild()
+        self.load_qt_database()
+        self.display_card_table()
 
     def load_qt_database(self):
         self.database().release_connection()
@@ -576,20 +623,20 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             sys.exit(1)
 
     def unload_qt_database(self):
+        # Don't save state twice when closing dialog.
+        if self.card_model is None:
+            return
         self.saved_index = self.table.indexAt(QtCore.QPoint(0,0))
         self.saved_selection = self.table.selectionModel().selectedRows()
-        if not self.config()["start_card_browser_sorted"]:
-            self.table.horizontalHeader().setSortIndicator\
-                (-1, QtCore.Qt.AscendingOrder)
         self.config()["browse_cards_dlg_table_settings"] \
             = self.table.horizontalHeader().saveState()
         self.table.setModel(QtGui.QStandardItemModel())
         del self.card_model
+        self.card_model = None
         QtSql.QSqlDatabase.removeDatabase(\
             QtSql.QSqlDatabase.database().connectionName())
 
     def display_card_table(self, run_filter=True):
-        self.load_qt_database()
         self.card_model = CardModel(self.component_manager)
         self.card_model.setTable("cards")
         headers = {QUESTION: _("Question"), ANSWER: _("Answer"),
@@ -621,10 +668,6 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             pass
         self.table.doubleClicked.connect(self.menu_edit)
         self.table.verticalHeader().hide()
-        for column in (_ID, ID, CARD_TYPE_ID, _FACT_ID, FACT_VIEW_ID,
-            ACQ_REPS_SINCE_LAPSE, RET_REPS_SINCE_LAPSE,
-            EXTRA_DATA, ACTIVE, SCHEDULER_DATA):
-            self.table.setColumnHidden(column, True)
         query = QtSql.QSqlQuery("select count() from tags")
         query.first()
         self.tag_count = query.value(0).toInt()[0]
@@ -649,9 +692,14 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
                     self.table.selectRow(index.row())
             self.table.setSelectionMode(old_selection_mode)
 
+    def reload_database_and_redraw(self):
+        self.load_qt_database()
+        self.display_card_table()
+
     def horizontal_header_section_clicked(self, index):
         if not self.config()["browse_cards_dlg_sorting_warning_shown"]:
-            self.main_widget().show_information(_("You chose to sort this table. Operations in the card browser could now be slower. Next time you start the card browser, the table will be unsorted again."))
+            self.main_widget().show_information(\
+_("You chose to sort this table. Operations in the card browser could now be slower. Next time you start the card browser, the table will be unsorted again."))
             self.config()["browse_cards_dlg_sorting_warning_shown"] = True
 
     def activate(self):
@@ -738,11 +786,17 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
             self.splitter_1.saveState()
         self.config()["browse_cards_dlg_splitter_2_state"] = \
             self.splitter_2.saveState()
+        # Make sure we start unsorted again next time.
+        if not self.config()["start_card_browser_sorted"]:
+            self.table.horizontalHeader().setSortIndicator\
+                (-1, QtCore.Qt.AscendingOrder)
 
     def closeEvent(self, event):
         # Generated when clicking the window's close button.
         self._store_state()
         self.unload_qt_database()
+        # This allows the state of the tag tree to be saved.
+        self.tag_tree_wdgt.close()
 
     def reject(self):
         # Generated when pressing escape.
@@ -754,5 +808,3 @@ class BrowseCardsDlg(QtGui.QDialog, Ui_BrowseCardsDlg, BrowseCardsDialog,
         self._store_state()
         self.unload_qt_database()
         return QtGui.QDialog.accept(self)
-
-
