@@ -30,12 +30,14 @@ class EditCardDlg(QtGui.QDialog, Ui_EditCardDlg, AddEditCards,
         return False
 
     def __init__(self, card, component_manager, allow_cancel=True,
-                 started_from_card_browser=False):
+                 started_from_card_browser=False, parent=None):
         # Note: even though this is in essence an EditFactDlg, we don't use
         # 'fact' as argument, as 'fact' does not know anything about card
         # types.
         AddEditCards.__init__(self, component_manager)
-        QtGui.QDialog.__init__(self, self.main_widget())
+        if parent is None:
+            parent = self.main_widget()
+        QtGui.QDialog.__init__(self, parent)
         self.setupUi(self)
         self.setWindowFlags(self.windowFlags() \
             | QtCore.Qt.WindowMinMaxButtonsHint)
@@ -129,11 +131,18 @@ class EditCardDlg(QtGui.QDialog, Ui_EditCardDlg, AddEditCards,
             unicode(self.tags.currentText()).split(',')]
         new_card_type_name = unicode(self.card_types_widget.currentText())
         new_card_type = self.card_type_by_name[new_card_type_name]
+        if new_fact_data == self.card.fact.data and \
+            ", ".join(new_tag_names) == self.card.tag_string() and \
+            new_card_type == self.card.card_type:
+                # No need to update the dialog.
+                QtGui.QDialog.reject(self)
+                return
+        # If this is called from the card browser, call this hook to unload
+        # the Qt database.
         if self.before_apply_hook:
             self.before_apply_hook()
-        status = self.controller().edit_sister_cards(self.card.fact,
-            new_fact_data, self.card.card_type, new_card_type, new_tag_names,
-            self.correspondence)
+        status = self.controller().edit_card_and_sisters(self.card, 
+            new_fact_data, new_card_type, new_tag_names, self.correspondence)
         if status == 0:
             tag_text = ", ".join(new_tag_names)
             self.config()["last_used_tags_for_card_type_id"][new_card_type.id] \
@@ -142,13 +151,18 @@ class EditCardDlg(QtGui.QDialog, Ui_EditCardDlg, AddEditCards,
             QtGui.QDialog.accept(self)
 
     def reject(self):  # Override 'add cards' behaviour.
-        if self.card_type_widget.fact_data() != self.card.fact.data:
-            status = QtGui.QMessageBox.warning(None, _("Mnemosyne"),
-                _("Abandon changes to current card?"),
-                _("&Yes"), _("&No"), "", 1, -1)
+        changed = False
+        for fact_key in self.card.fact.data:
+            if fact_key in self.card_type_widget.fact_data() and \
+                self.card_type_widget.fact_data()[fact_key] \
+                != self.card.fact.data[fact_key]:
+                changed = True
+                break
+        if changed:
+            status = self.main_widget().show_question(\
+                _("Abandon changes to current card?"), _("&Yes"), _("&No"), "")
             if status == 0:
                 QtGui.QDialog.reject(self)
                 return
         else:
            QtGui.QDialog.reject(self)
-
