@@ -135,8 +135,17 @@ class Client(Partner):
             if self.check_for_edited_local_media_files:
                 self.ui.set_progress_text("Checking for edited media files...")
                 self.database.check_for_edited_media_files()
-            socket.setdefaulttimeout(30)
+                self.ui.set_progress_text("Dynamically creating media files...")
+                self.database.dynamically_create_media_files()
+            # Set timeout long enough for e.g. a slow NAS waking from 
+            # hibernation.
+            socket.setdefaulttimeout(45)
             self.login(username, password)
+            # Generating media files at the server side could take some time,
+            # so we update the timeout.
+            self.con = None
+            socket.setdefaulttimeout(15*60)
+            self.get_server_check_media_files()
             # Do a full sync after either the client or the server has restored
             # from a backup.
             if self.database.is_sync_reset_needed(\
@@ -151,6 +160,11 @@ class Client(Partner):
                 else:
                     self.get_server_entire_database()
                 self.get_sync_finish()
+                # Fetch config settings.
+                self.login(username, password)
+                self.get_server_generate_log_entries_for_settings()
+                self.get_server_log_entries()
+                self.get_sync_finish()
             # First sync, put binary database to server if supported.
             elif not self.database.is_empty() and \
                     self.server_info["is_database_empty"] and \
@@ -158,6 +172,12 @@ class Client(Partner):
                 self.put_client_media_files(reupload_all=True)
                 self.put_client_entire_database_binary()
                 self.get_sync_finish()
+                # Upload config settings.
+                self.login(username, password)
+                self.database.generate_log_entries_for_settings()
+                self.put_client_log_entries()
+                self.get_server_log_entries()
+                self.get_sync_finish()
             else:
                 # Upload local changes and check for conflicts.
                 result = self.put_client_log_entries()
@@ -192,7 +212,8 @@ class Client(Partner):
                 # sync next time.
                 self.ui.show_error("Sync failed, restoring from backup. " + \
                     "The next sync will need to be a full sync.")
-                self.database.restore(backup_file)
+                if backup_file:
+                    self.database.restore(backup_file)
         finally:
             if self.con:
                 self.con.close()
@@ -208,9 +229,11 @@ class Client(Partner):
         if restored_from_backup:
             message = "The database was restored from a backup, either " + \
                 "automatically because of an aborted sync or manually by " + \
-                "the user.\nFor safety, a full sync needs to happen.\n"
+                "the user.\nFor safety, a full sync needs to happen and " + \
+                "you need to choose which copy of the database to keep and " + \
+                "which copy to discard.\n"
         else:
-            message = "Conflicts detected during sync!"
+            message = "Conflicts detected during sync! Choose which version to keep and which version to discard."
         # Ask for conflict resolution direction.
         if self.supports_binary_upload():
             result = self.ui.show_question(message,
@@ -218,7 +241,7 @@ class Client(Partner):
             results = {0: "KEEP_LOCAL", 1: "KEEP_REMOTE", 2: "CANCEL"}
             result = results[result]
         else:
-            message += "Your client only stores part of the server " + \
+            message += " " + "Your client only stores part of the server " + \
                 "database or uses a different software version, " + \
                 "so you can only fetch the remote version."
             result = self.ui.show_question(message,
@@ -321,6 +344,15 @@ class Client(Partner):
             self.server_info["machine_id"])
         self.database.merge_partners(self.server_info["partners"])
 
+    def get_server_check_media_files(self):
+        self.ui.set_progress_text("Asking server to check for updated media files...")
+        self.request_connection()
+        self.con.request("GET", self.url(\
+            "/server_check_media_files?" + \
+            "session_token=%s" % (self.server_info["session_token"], )))
+        response = self.con.getresponse()
+        self._check_response_for_errors(response, can_consume_response=True)
+
     def put_client_log_entries(self):
 
         """Contrary to binary files, the size of the log is not known until we
@@ -458,6 +490,15 @@ class Client(Partner):
             self.server_info["machine_id"])
         self.database.remove_partnership_with(self.machine_id)
 
+    def get_server_generate_log_entries_for_settings(self):
+        self.ui.set_progress_text("Getting settings...")
+        self.request_connection()
+        self.con.request("GET", self.url(\
+            "/server_generate_log_entries_for_settings?" + \
+            "session_token=%s" % (self.server_info["session_token"], )))
+        response = self.con.getresponse()
+        self._check_response_for_errors(response, can_consume_response=True)
+
     def put_client_media_files(self, reupload_all=False):
         self.ui.set_progress_text("Sending media files...")
         if reupload_all:
